#
# spec file for package parsec
#
# Copyright (c) 2022 SUSE LLC
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


%global rustflags '-Clink-arg=-Wl,-z,relro,-z,now'
%define archive_version 1.0.0

%{?systemd_ordering}
Name:           parsec
Version:        1.0.0
Release:        bp154.1.13
Summary:        Platform AbstRaction for SECurity
License:        Apache-2.0
URL:            https://parallaxsecond.github.io/parsec-book
Source0:        https://github.com/parallaxsecond/parsec/archive/%{archive_version}.tar.gz#/parsec-%{archive_version}.tar.gz
Source1:        vendor.tar.xz
Source2:        cargo_config
Source3:        parsec.service
Source4:        config.toml
Source5:        parsec.conf
Source6:        system-user-parsec.conf
Source10:       https://git.trustedfirmware.org/TS/trusted-services.git/snapshot/trusted-services-389b506.tar.gz
BuildRequires:  cargo
BuildRequires:  clang-devel
BuildRequires:  cmake
BuildRequires:  llvm-devel
BuildRequires:  pkgconfig
BuildRequires:  protobuf-devel
BuildRequires:  python3
# jwt-svid-authenticator (SPIFFE-based authenticator) needs rust >= 1.53
BuildRequires:  rust >= 1.53
BuildRequires:  rust-packaging
BuildRequires:  sysuser-tools
BuildRequires:  pkgconfig(tss2-esys) >= 2.3.3
# opensc is used to initialize HSM keys (PKCS#11 backend)
Recommends:     opensc
%sysusers_requires
# /dev/tpm* are owned by tss user
Requires(pre):  system-user-tss
# tpm2-0-tss holds the udev rule to make /dev/tpm* owned by tss user
Requires:       tpm2-0-tss
# Without libtss2-tcti-device0 parsec fails to start TPM properly
Requires:       libtss2-tcti-device0
ExcludeArch:    %{ix86} armv6l armv6hl

%description
PARSEC is the Platform AbstRaction for SECurity, an open-source initiative to provide
a common API to hardware security and cryptographic services in a platform-agnostic way.
This abstraction layer keeps workloads decoupled from physical platform details,
enabling cloud-native delivery flows within the data center and at the edge.

%prep
%setup -q -a1 -a10 -n parsec-%{archive_version}
rmdir trusted-services-vendor
mv trusted-services-389b506 trusted-services-vendor
rm -rf .cargo && mkdir .cargo
cp %{SOURCE2} .cargo/config
# Enable all providers
sed -i -e 's#default = \["unix-peer-credentials-authenticator"\]##' Cargo.toml
# Features available in 1.0.0:
# all-providers = ["tpm-provider", "pkcs11-provider", "mbed-crypto-provider", "trusted-service-provider"]
# all-authenticators = ["direct-authenticator", "unix-peer-credentials-authenticator", "jwt-svid-authenticator"]
# But disable "trusted-service-provider" until we have a trusted-services package
echo 'default = ["tpm-provider", "pkcs11-provider", "mbed-crypto-provider", "all-authenticators"]' >> Cargo.toml

%build
export PROTOC=%{_bindir}/protoc
export PROTOC_INCLUDE=%{_includedir}
%cargo_build
%sysusers_generate_pre %{SOURCE6} parsec

%install
export PROTOC=%{_bindir}/protoc
export PROTOC_INCLUDE=%{_includedir}
%cargo_install
mkdir -p %{buildroot}%{_sysusersdir}
install -m 0644 %{SOURCE6} %{buildroot}%{_sysusersdir}/
install -D -p -m0644 %{SOURCE3} %{buildroot}%{_unitdir}/parsec.service
install -D -p -m0644 %{SOURCE4} %{buildroot}%{_sysconfdir}/parsec/config.toml
install -D -p -m0644 %{SOURCE5} %{buildroot}%{_tmpfilesdir}/parsec.conf
install -d -m0755 %{buildroot}%{_localstatedir}/lib/parsec
# Move parsec to _libexecdir
mkdir -p %{buildroot}%{_libexecdir}
mv target/release/parsec %{buildroot}%{_libexecdir}
# Clean-up
find %{buildroot} -name .crates2.json -delete
rm -rf %{buildroot}%{_datadir}/cargo/registry

%check
export PROTOC=%{_bindir}/protoc
export PROTOC_INCLUDE=%{_includedir}
%cargo_test -- --lib

%pre -f parsec.pre
%service_add_pre parsec.service

%post
%service_add_post parsec.service
%tmpfiles_create %_tmpfilesdir/parsec.conf

%preun
%service_del_preun parsec.service

%postun
%service_del_postun parsec.service

%files
%license LICENSE
%doc README.md config.toml
%attr(0750,parsec,parsec) %dir %{_sysconfdir}/parsec/
%attr(0750,parsec,parsec) %dir %{_localstatedir}/lib/parsec/
%attr(0644,parsec,parsec) %config(noreplace) %{_sysconfdir}/parsec/config.toml
%{_libexecdir}/parsec
%{_tmpfilesdir}/parsec.conf
%{_unitdir}/parsec.service
%{_sysusersdir}/system-user-parsec.conf

%changelog
* Thu Mar 31 2022 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Update to 1.0.0:
  * Update changelog file only
- Disable build on x86 since the build fails
* Mon Mar 21 2022 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Update to 1.0.0-rc3:
  * Changelog: https://github.com/parallaxsecond/parsec/compare/1.0.0-rc2...1.0.0-rc3
  * Fixes CVE-2022-24713 - boo#1196972
* Tue Mar  8 2022 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Update to 1.0.0-rc2:
  * Changelog: https://github.com/parallaxsecond/parsec/compare/1.0.0-rc1...1.0.0-rc2
- Remove CryptoAuthLib (CAL) provider as it is unmaintained.
* Thu Feb 24 2022 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Update to 1.0.0-rc1:
  * Changelog: https://github.com/parallaxsecond/parsec/compare/0.8.1...1.0.0-rc1
- Drop upstream patch:
  * harden_parsec.service.patch
* Thu Jan 13 2022 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Update to 0.8.1:
  * Changelog: https://github.com/parallaxsecond/parsec/compare/0.8.0...0.8.1
* Thu Dec  9 2021 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Fix /run/parsec permission to 755. This is enough for all
  users to access the service - boo#1193484 - CVE-2021-36781
* Fri Oct 15 2021 Johannes Segitz <jsegitz@suse.com>
- Added hardening to systemd service(s) (bsc#1181400). Added patch(es):
  * harden_parsec.service.patch
  Modified:
  * parsec.service
  * Upstream submission: https://github.com/parallaxsecond/parsec/issues/569
* Mon Sep 27 2021 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Allow access to /run/parsec to all users
- Requires rust 1.53+ (now also available in 15.3/15-SP3 via Update)
  which allow to enable same features accross distros
* Fri Aug  6 2021 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Update to 0.8.0:
  * Changelog: https://github.com/parallaxsecond/parsec/compare/0.7.2...0.8.0
- Drop upstream patch:
  * parsec-fix-old-rust.patch
- Disable 'trusted-service-provider' as it currently fails to build
- Disable 'jwt-svid-authenticator' (SPIFFE-based authenticator)
  on Leap, as it cannot be compiled with rust 1.43.1
* Fri Apr 30 2021 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Fix build with old rust used in Leap 15.3/SLE15-SP3.
  * parsec-fix-old-rust.patch
* Wed Apr 14 2021 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Requires 'tpm2-0-tss' as it holds the udev rule to make /dev/tpm*
  owned by tss user
- Requires 'libtss2-tcti-device0' as it is required to start parsec
  with TPM support
* Wed Apr  7 2021 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Enable all authenticators which adds 'direct-authenticator'
- Add CryptoAuthLib template (disabled) in config.toml
* Thu Apr  1 2021 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Make 'parsec' user part of 'tss' group to access /dev/tpm* devices
* Wed Mar 31 2021 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Update to 0.7.2:
  * Changelog: https://github.com/parallaxsecond/parsec/compare/0.6.0...0.7.2
* Tue Mar 30 2021 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Recommends 'opensc' as it is used to initialize HSM keys (PKCS#11 backend)
- Add PKCS#11 template (disabled) in config.toml
- Disable Tpm backend by default in config.toml
* Fri Nov 13 2020 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Enable 'all-providers' which adds 'pkcs11-provider' compared to
  previous config
- Drop unneeded patch:
  * pkcs11-libloading-issue.patch
* Fri Oct 23 2020 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Update to 0.6.0
- Update config.toml
- Add patch:
  * pkcs11-libloading-issue.patch
* Fri Oct 16 2020 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Initiale version 0.5.0
